//=========================================================================
//
//	FILE:				CP_MacOSHandle.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to provide a wrapper around a MacOS Handle.
//						This class can also be used if QuickTime for Windows
//						is defined.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_MacOSHandle
#define _H_CP_MacOSHandle
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_M_ReferenceCounted.h"
#include "CP_Types.h"

#if TARGET_OS_MAC || CP_Optional_QuickTime_Windows
	#if TARGET_API_MAC_CARBON && __CF_USE_FRAMEWORK_INCLUDES__
		#include <Carbon/Carbon.h>
	#else
		#include <MacMemory.h>
	#endif
#endif

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_MAC || CP_Optional_QuickTime_Windows
/*!
*	@class		CP_MacOSHandle
*	@brief		A class to provide a wrapper around a MacOS Handle. This class can
*				also be used if QuickTime for Windows is defined.
*/
class CP_EXPORT CP_MacOSHandle : public CP_M_RefCounted
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MacOSHandle();
										CP_MacOSHandle( UInt32 inSize );
										CP_MacOSHandle( CP_MacOSHandle& inHandleBlock );
										CP_MacOSHandle(
												Handle inMacHandle,
												bool inOwnsHandle = true );
	//! Destructor.
										~CP_MacOSHandle() throw();
	//@}

	//! @name Copying
	//@{
			void						CopyTo(
												void *inBuffer,
												UInt32 inNumBytes );
			void						CopyFrom(
												void *ioBuffer,
												UInt32 inNumBytes );
	//@}

	//! @name Getters
	//@{
			Handle						GetHandle();
			char						*GetPtr();
	//@}

	//! @name Length
	//@{
			UInt32						Length() const;
			void						SetLength( UInt32 inLength );
	//@}
			
	//! @name Locking
	//@{
			void						Lock();
			void						Unlock();

										/*!
										*	Returns true if the memory block is locked.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if the memory block is locked.
										*/
			bool						IsLocked()									{ return( fLocked ); }
	//@}

	//! @name Operators
	//@{
			CP_MacOSHandle&				operator = (CP_MacOSHandle& inHandleBlock );

										/*!
										*	Returns the MacOS Handle.
										*	
										*	@exception	none
										*	
										*	@return		Handle
										*/
										operator Handle()							{ return( fBlock ); }
	//@}
			
	//! @name Ownership
	//@{
			void						Adopt( Handle inHandle );
			Handle 						Release() const;
	//@}

	//! @name Testers
	//@{
										/*!
										*	Returns true if the memory block is not NULL.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if the memory block is not NULL.
										*/
			bool						IsValid()									{ return( fBlock != NULL ); }

										/*!
										*	Returns true if this object owns the block of memory, and is therefore
										*	responsible for its deletion. Otherwise the memory block held by this object
										*	will not be deleted by this class.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if this object owns the memory handle.
										*/
			bool						OwnsHandle()								{ return( fOwnsHandle ); }
	//@}
			
	//! @name Utility
	//@{
			void						Wrap(
												Handle inExistingHandle,
												bool inOwnsHandle = true );


										/*!
										*	Notifies this class that it no longer owns the memory, and will not
										*	delete it. You are now responsible for deleting the memory to avoid a
										*	memory leak.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						Detach()									{ fOwnsHandle = false; }
							
			CP_MacOSHandle				*Clone();
	//@}

private:
	//! @name Allocation
	//@{
			void						Allocate( UInt32 inSize );
			void						Deallocate();
	//@}

	// class data
protected:
	Handle								fBlock;				/*! Platforms memory object.*/
	mutable bool						fOwnsHandle;		/*! Flag to indicate who is responsible for deleting the platforms memory.*/
	bool								fLocked;			/*! Flag to indicate if the block is locked.*/
	UInt32								fActualBlockLength;	/*! Length of the memory block.*/
};

#endif	// TARGET_OS_MAC || CP_Optional_QuickTime_Windows

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_MacOSHandle


