//=========================================================================
//
//	FILE:				CP_Parser.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_Parser
#define _H_CP_Parser
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_String.h"
#include "CP_Types.h"
#include "CP_Utility.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

//========================================================================
// CP_ParsedField
//		Fields from our parsed string created in the CP_Parse object.
//========================================================================
//!  CP_ParsedField
/*!
  Fields from our parsed string created with the CP_Parse object.
*/
class CP_EXPORT CP_ParsedField
{
public:
											// Construction / Destruction
										/*! Constructor. */
										CP_ParsedField()									{}

										/*! Constructor with a CP_String containing the data for this field.
										\param inData - contains the parsed fields data. */
										CP_ParsedField( const CP_String& inData ) 			{ fFieldData = inData; }

										/*! Copy constructor.
										\param inField - the field we are copy the data for. */
										CP_ParsedField( const CP_ParsedField& inField ) 	{ fFieldData = inField.fFieldData; }

										/*! Destructor. */
	virtual								~CP_ParsedField();

										/*! Return the fields data as a SInt16. */
										operator SInt16() 									{ return CP_Utility::CP_StringToSInt16( fFieldData ); }
										/*! Return the fields data as a SInt32. */
										operator SInt32()									{ return CP_Utility::CP_StringToSInt32( fFieldData ); }
										/*! Return the fields data as a double. */
										operator double()									{ return CP_Utility::CP_StringToDouble( fFieldData ); }
										/*! Return the fields data as a float. */
										operator float()									{ return CP_Utility::CP_StringToFloat( fFieldData ); }

										/*! Assignment operator.
										\param inString - contains the string data to assign to this field. */
			CP_ParsedField&  			operator =( CP_String& inString )					{ fFieldData = inString; return( *this ); };

										/*! Assignment operator.
										\param inField - contains the field we are going to copy from. */
			CP_ParsedField&  			operator =( CP_ParsedField& inField ) 				{ fFieldData = inField.fFieldData; return( *this ); };

										/*! Equality operator. Returns true if this object contains the same data as inField.
										\param inField - the field we are checking against. */
			bool			  			operator ==( const CP_ParsedField& inField ) 		{ return( fFieldData == inField.fFieldData ); };

										/*! Return the fields data. */
			CP_String					GetStringData()										{ return( fFieldData ); }

	// class data
protected:
										/*! CP_String containing the fields data. */
	CP_String							fFieldData;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

#if ! TARGET_OS_LINUX
	typedef std::vector<CP_ParsedField *, std::allocator<CP_ParsedField *> >						CP_ParsedField_Array;
	typedef std::vector<CP_ParsedField *, std::allocator<CP_ParsedField *> >::iterator				CP_ParsedField_Iterator;
	typedef std::vector<CP_ParsedField *, std::allocator<CP_ParsedField *> >::reverse_iterator		CP_ParsedField_ReverseIterator;
#else
	typedef std::vector<CP_ParsedField *>															CP_ParsedField_Array;
	typedef std::vector<CP_ParsedField *>::iterator													CP_ParsedField_Iterator;
	typedef std::vector<CP_ParsedField *>::reverse_iterator											CP_ParsedField_ReverseIterator;
#endif

//!  CP_Parser
/*!
  A class to parse simple data into seperate fields. For example: this class can be used to seperate
  comma or tab deleted file data, string data that is seperated by commas, or formatted dates, etc.
*/
class CP_EXPORT CP_Parser
{
public:
											// Construction / Destruction
										CP_Parser( bool inStripQuotes = false );
										CP_Parser(
												const CP_String& inData,
												const CP_String& inSeperator = ",",
												bool inStripQuotes = false );
										CP_Parser(
												const CP_FileSystemSpec& inFile,
												const CP_String& inSeperator = ",",
												bool inStripQuotes = false );
		virtual							~CP_Parser();

										/*! Return a CP_String containing the list of seperators that can be used with
										the data string for this class. */
				CP_String				GetSeperator()										{ return( fSeperator ); }
										/*! Sets the seperators that can be recognized between field data. */
				void					SetSeperator( const CP_String& inSeperator )		{ fSeperator = inSeperator; }

				bool					Parse();
				bool					Parse(
												const CP_String& inData,
												const CP_String& inSeperator = ",",
												bool inStripQuotes = false );
				bool					ParseFile(
												const CP_FileSystemSpec& inFileSpec,
												const CP_String& inSeperator = "," );

										/*! Return the number of fields found. */
				UInt32					GetCount()											{ return( fFields.size() ); }
				bool					GetFieldData(
												UInt32 inFieldIndex,
												CP_String& inFieldData );
				bool					GetFieldData(
												UInt32 inFieldIndex,
												char *inBuffer,
												SInt32 inMaxBufferLength,
												SInt32 *inOutActualDataLength );
				CP_ParsedField			*GetField( UInt32 inFieldIndex );
	
				void					Reset();


protected:           
				bool					IsSeperator( char inchar );
				bool					LoadFile( const CP_FileSystemSpec& inFileSpec );

	// class data
protected:
										/*! List of the fields found. */
	CP_ParsedField_Array				fFields;		// list of CP_ParsedFields
										/*! Flag to indicate how quotes are handled. */
	bool 								fStripQuotes;	// flag to indicate how quotes are handled
										/*! A CP_String containing all of the valid seperators for the data to be parsed. */
	CP_String							fSeperator;		// seperators used between fields
										/*! A CP_String containing the data to parse. */
	CP_String							fData;			// data to parse
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Parser
