//=========================================================================
//
//	FILE:				CP_TimeZone.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Representation of a time zone.
//	
//	HISTORY:			1st Version - December 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 107 $
//						Latest Checkout:  $Date: 2005-03-21 15:11:37 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_TimeZone
#define _H_CP_TimeZone

#include "CP_Object.h"
#include "CP_CFObject.h"
#include "CP_String.h"

CPLAT_Begin_Namespace_CPLAT

class CP_DateTime;

/*! A class holding information about a speific time zone.
	\sa CP_DateTime
*/
class CP_EXPORT CP_TimeZone : public CP_Object, public CP_M_RefCounted
#if TARGET_OS_MAC
	, public CP_CFObject<CFTimeZoneRef>
#endif
{
public:
	//! @name Accessing Shared Timezones
	//@{
	/*!
	 * Returns the default user time zone. Defaults to the system time zone
	 * unless the user set it something else using \ref CP_TimeZone::SetDefaultZone().
	 *
	 * @exception none
	 *
	 * @result CP_TimeZone*		A pointer to the default user time zone.
	 */
	static const CP_TimeZone* DefaultZone();
	/*!
	 * Returns the time zone currently used by the system.
	 *
	 * @exception none
	 *
	 * @result CP_TimeZone*		A pointer to the time zone currently used by the system.
	 */
	static const CP_TimeZone* SystemZone();
	/*!
	 * Returns the GMT (Greenwich Mean Time) time zone.
	 *
	 * @exception none
	 *
	 * @result CP_TimeZone*		A pointer to the time zone representing GMT.
	 */
	static const CP_TimeZone* GMTZone();
	//@
	
	//! @name Constructor / Destructor
	//@{
	/*!
	 * Constructs a timezone by looking up the timezone data using the specified name
	 * which represents a geographical location or abbreviation.
	 * \sa CP_TimeZone::GetKnownNames
	 *
	 * @exception std::invalid_argument	raised if no time zone with the given name could be found.
	 *
	 * @param inName			The name of the geographical location or an abbreviation of a timezone name.
	 * @param inSearchAbbrev	If \a true, tries to resolve \a inName using the list of known abbreviations before
	 *                          looking for a zone with matching name.
	 *							If \a false, it only looks for matching names.
	 */
	CP_TimeZone(const CP_String& inName, bool inSearchAbbrev);
	/*!
	 * Construct a custom timezone with the specified GMT (Greenwich Mean Time) offset. 
	 * \note The new timezone won't be found using \ref CP_TimeZone(const CP_String&, bool) even
	 *       when passing the same name as the name is not registered as a known location.\n
	 *       Also, this time zone will never take into account daylight saving time, as it doesn't 
	 *       know about the rules.
	 *
	 * @exception none.
	 *
	 * @param inSecondsFromGMT	Time offset from GMT in seconds (negativ numbers indicate timezone is behind GMT).
	 * @param inName			The name for the custom timezone.
	 */
	CP_TimeZone(SInt32 inSecondsFromGMT, const CP_String& inName);
	//! Destructor.
	virtual ~CP_TimeZone() throw();
	//@}
	
	/*!
	 * Returns whether daylight saving time was active at the specified date.
	 *
	 * @exception none
	 *
	 * @result bool		Was daylight saving time in action at the specified date?
	 */
	bool IsDaylightSavingTime(const CP_DateTime& inDate) const;
	/*!
	 * Returns the time zone's name, which is usually the geographical region name for the
	 * standard time zones or the custom name supplied in the constructor for custom timezones.
	 *
	 * @exception none
	 *
	 * @result CP_String	The time zone's name.
	 */
	CP_String GetName() const;
	/*!
	 * Returns the time zone's abbreviation at the specified date.
	 * \note The abbreviation may be different at different dates. For example, during daylight savings time 
	 *       the US/Eastern time zone has an abbreviation of "EDT." At other times, its abbreviation is "EST."
	 *
	 * @exception none
	 *
	 * @result CP_String	The time zone's abbreviation at the specified date.
	 */
	CP_String GetAbbreviation(const CP_DateTime& inDate) const;
	/*!
	 * Returns the difference between the time zone and GMT (Greenwich Mean Time) at the specified date. 
	 *
	 * @exception none
	 *
	 * @result SInt32	The time difference between the time zone and GMT in seconds.
	 */
	SInt32 GetSecondsFromGMT(const CP_DateTime& inDate) const;
	
	/*!
	 * Returns a list of known timezone names. 
	 *
	 * @exception none
	 *
	 * @param[out] outAllNames	A vector of strings that will contain the names on exit.
	 */
	static void GetKnownNames(std::vector<CP_String>& outAllNames);

	/*!
	 * Sets the default user time zone. This is the time zone returned by \ref CP_TimeZone::DefaultZone(). 
	 *
	 * @exception none
	 *
	 * @param inDefaultZone		A pointer to a time zone object to use as the default zone, or \a NULL to use the system time zone.
	 */
	static void SetDefaultZone(const CP_TimeZone* inDefaultZone);

	//========================================================================
	// Platform specific routines
	//========================================================================
#if TARGET_OS_MAC
public:
	CP_TimeZone(CFTimeZoneRef inTimeZoneRef, bool inRetain);
#endif
#if TARGET_OS_WIN32
	bool IsDaylightSavingTime( const SYSTEMTIME& inDate ) const;
	SInt32 GetSecondsFromGMT( const SYSTEMTIME& inDate ) const;
	CP_TimeZone();
#endif
#if TARGET_OS_LINUX
#endif
		
	// class data:
private:
	static CP_SmartRef<const CP_TimeZone>		gDefaultZone;

#if TARGET_OS_WIN32
	TIME_ZONE_INFORMATION fTimeZoneInfo;   // keep the time zone information like name and daylight dates
	
	static std::map<CP_String, TIME_ZONE_INFORMATION> gRegisteredZones;
	static bool gRegisteredZonesInitialized;
#endif
};

CPLAT_End_Namespace_CPLAT

#endif // _H_CP_TimeZone

