//=========================================================================
//
//	FILE:				CP_Document_Base.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class to handle management of documents. A document
//						associates a CP_Document_BaseWindow and CP_File so the
//						contents of the window can be written to the file.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 218 $
//						Latest Checkout:  $Date: 2005-06-22 23:43:56 -0700 (Wed, 22 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_Document_Base
#define _H_CP_Document_Base
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_AskRevertDialog.h"
#include "CP_AskSaveChangesDialog.h"
#include "CP_DocumentWindow.h"
#include "CP_M_CommandHandler.h"
#include "CP_OSIcon.h"
#include "CP_Persistant.h"
#include "CP_PutFileDialog.h"
#include "CP_ScriptableObject.h"
#include "CP_SignalSlot.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Document_Base
*	@brief		Base class to manage a single document.
*	
*				This is the main or base document handling class to use in your application
*				when you need documents support. This class is also used with the CP_Document_BaseApp
*				class instead of the CP_Application class.
*
*				A CP_Document_Base associates a CP_Document_BaseWindow and a CP_File so that the
*				data in the window can be written out as a persistant state.
*
*				This class inherits from CP_Persistant which has method you can override to handle
*				opening and saving the contents of this class.
*
*				In order for the relationship between the CP_Document_BaseWindow class and this
*				document class to handle saving of the windows contents automatically you need to
*				ensure you have only 1 CP_Document_BaseWindow associated with this class.
*
*	@group		Documents
*/
class CP_EXPORT CP_Document_Base : public CP_ScriptableObject, public CP_Persistant, public CP_M_CommandHandler
{
public:
	enum
	{
		CP_E_Doc_Create_DoNothing = 1,
		CP_E_Doc_Create_CreateUI = 2,
		CP_E_Doc_Create_CreateUIOnNew = 3,
		CP_E_Doc_Create_CreateUIOnOpen = 4
	};

	friend class CP_DocumentNameMatch;

	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Document_Base(
												CP_ScriptableObject *inContainer,
												SInt32 inDocType );
	//! Destructor.
	virtual								~CP_Document_Base() throw();
	//@}
				
	//! @name Persistance
	//@{
	static CP_Document_Base				*DocumentCreator(
												CP_ScriptableObject *inContainer,
												SInt32 inDocumentClass );
	//@}

	//! @name Closing
	//@{
	virtual	void						AttemptCloseDocument();
			void						CloseDocument();
	//@}

	//! @name Getters
	//@{
	virtual	CP_EventHandler&			GetEventHandler();

	virtual	FourCharCode				GetDefaultFileCreator()						{ return( fDefaultFileCreator ); }
	virtual	CP_String					GetFileName() const;
	virtual	CP_String					GetFileExtension() const;
	virtual	FourCharCode				GetDefaualtFileType()						{ return( fDefaultFileType ); }
	virtual	CP_OSIcon					*GetIcon();
	virtual	CP_String					GetDefaultFileName( const CP_String& inFileName ) const;
	//@}

	//! @name Opening
	//@{
	virtual	void						HandleOpen( const CP_FileSystemSpec& inFileSpec );
	//@}

	//! @name Persist
	//@{
	virtual	void						DoPostCreate( SInt32 inPostCreateFlags = CP_E_Doc_Create_DoNothing );
	//@}

	//! @name Saving Commands
	//@{
	virtual	bool						HandleSave(
												const CP_FileSystemSpec& inFileSpec,
												FourCharCode inCreator,
												FourCharCode inType,
												const CP_String& inFileExtension,
												bool inReplaceFile );
	virtual	void						HandleRevert();

	virtual	void						HandleSaveCommandStatus(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommand );
	virtual	void						HandleSaveCommand( OSStatus& inOutStatus );
	virtual	void						HandleSaveAsCommandStatus(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommand );
	virtual	void						HandleSaveAsCommand( OSStatus& inOutStatus );
	virtual	void						HandleRevertCommandStatus(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommand );
	virtual	void						HandleRevertCommand( OSStatus& inOutStatus );
	//@}

	//! @name Undo commands
	//@{
	virtual	void						HandleUndoCommandStatus(
												CP_CommandStatus& /*inOutCommandStatus*/,
												bool& /*inOutResult*/,
												CP_CommandNo& /*inCommand*/ )				{}
	virtual	void						HandleUndoCommand(
												OSStatus& /*inOutStatus*/,
												CP_CommandNo& /*inCommand*/ )				{}
	virtual	void						HandleRedoCommandStatus(
												CP_CommandStatus& /*inOutCommandStatus*/,
												bool& /*inOutResult*/,
												CP_CommandNo& /*inCommand*/ )				{}
	virtual	void						HandleRedoCommand(
												OSStatus& /*inOutStatus*/,
												CP_CommandNo& /*inCommand*/ )				{}
	//@}

	//! @name Setters
	//@{
	virtual	void						SetDefaultFileCreator( FourCharCode inCreator )		{ fDefaultFileCreator = inCreator; }
	virtual	void						SetDefaualtFileType( FourCharCode inFileType )		{ fDefaultFileType = inFileType; }
	virtual	void						SetFileName( const CP_String& inFileName )			{ fFileName = inFileName; }
	virtual	void						SetFileExtension( const CP_String& inExtension )	{ fExtension = inExtension; }
	//@}

	//! @name UI
	//@{
										/*!
										*	User overrideable method to handle creation of the document
										*	main UI and windows.
										*
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual	void						HandleCreateDocumentUI( SInt32 inCreateFlags = CP_E_Doc_Create_DoNothing );
	virtual	void						Show();
	virtual	void						Hide();

										/*!
										*	Return the main document window.
										*
										*	@exception	none
										*	
										*	@return		CP_Document_BaseWindow *		The CP_Document_BaseWindow being used as the main window.
										*/
			CP_DocumentWindow			*GetMainWindow()									{ return( fMainWindow ); }
			void						SetMainWindow( CP_DocumentWindow *inMainWindow );
	//@}

	//! @name File Dialogs
	//@{
			CP_AskSaveChangesDialog		*GetSaveChangesDialog();
			CP_AskRevertDialog 			*GetAskRevertDialog();
			CP_PutFileDialog			*GetPutFileDialog();

										/*!
										*	Returns the file spec used for this documents main file.
										*
										*	@exception	none
										*	
										*	@return		CP_FileSystemSpec	The documents main files file system spec.
										*/
			CP_FileSystemSpec			GetFileSpec()										{ return( fFileSpec ); }

										/*!
										*	Sets the file spec used for this documents main file.
										*
										*	@param		inFileSpec			The file spec for this documents main file.
										*
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetFileSpec( const CP_FileSystemSpec& inFileSpec )	{ fFileSpec = inFileSpec; }

			bool						GetSaveIsSaveAs()									{ return( fSaveIsSaveAs ); }
	//@}

protected:
	//! @name Events
	//@{
										/*!
										*	User overrideable method to handle an Undo event.
										*
										*	@param		inEvent			Not used.
										*
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual	void						HandleUndoEvent( CP_Event_Base& /*inEvent*/ )		{}
	//@}

private:
	//! @name Signal / Slots
	//@{
			void						HandleSaveDocumentEvent( CP_Event_Base& inEvent );

			void						HandleAskSave_SaveChanges(
												CP_FileSystemDialogs_Base *inSendingDialog,
												CP_E_NavDialogResult inNavDialogResult );
			void						HandleAskSave_DontSaveChanges(
												CP_FileSystemDialogs_Base *inSendingDialog,
												CP_E_NavDialogResult inNavDialogResult );
			void						HandlePutFileDialogResult(
												CP_FileSystemDialogs_Base *inSendingDialog,
												const CP_FileSystemSpec& inSelection,
												bool inOverwrite );
			void						HandleRevert_DoRevert(
												CP_FileSystemDialogs_Base *inSendingDialog,
												CP_E_NavDialogResult inNavDialogResult );
	//@}

	// class data
protected:
	CP_DocumentWindow					*fMainWindow;			/*! This is the main window for this class, and allows communication
																	between the window and this document so closing the window will
																	also cause this document to close. */

	bool								fCloseInProgress;		/*! Flag used to indicate if a close is in progress. Used
																	with non-modal file dialogs to monitor progress. */
	
	bool								fSaveIsSaveAs;
	
	CP_EventHandler						fDummyHandler;			/*! Default event handler returnned for this class. */
	
	FourCharCode						fDefaultFileCreator;	/*! Default file creator used with documents. Users should provide
																	actual creator if different than the applications. */
	FourCharCode						fDefaultFileType;		/*! Defaualt file type code for documents created with this
																	class. Default is 'TEXT'. */
	
	CP_String							fFileName;				/*! File name for the default document of this class. */
	CP_String							fExtension;				/*! Extension used with the file for this document. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name AppleEvents
	//@{
	virtual	SInt32						CountContainedObjects( const DescType inType );

	virtual	CP_ScriptableObject_Imp		*GetContainedObjectByName(
												const DescType inDesiredClass,
												const CP_String& inName );
	virtual	CP_ScriptableObject_Imp		*GetContainedObjectByPosition(
												const DescType inDesiredClass,
												const SInt32 inPosition  );

	virtual	void						DoAE_GetData(
												const DescType inPropertyType,
												const AEDesc& inValue,
												AppleEvent& inOutReply );
	//@}

	//! @name ProxyIcon
	//@{
	void								LoadProxyIcon();
	//@}

protected:
	//! @name AppleEvents
	//@{
	virtual	DescType					GetAppleClass() const;
	virtual	void						MakeSelfSpecifier(
												AEDesc& inContainerSpec,
												AEDesc& inOutSelfSpec );
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark === CP_DocumentNameMatch ==
#endif

/*!
*	@class		CP_DocumentNameMatch
*	@brief		Class used in our document name search to compare document titles.
*/
class CP_DocumentNameMatch
{
public:
										/*!
										*	Constructor.
										*
										*	@param		inDocumentTitle		The document title to compare.
										*
										*	@exception	none
										*/
  										CP_DocumentNameMatch( const CP_String& inDocumentTitle )
  											{
  												fDocumentTitle = inDocumentTitle;
  											}
  						
										/*!
										*	operator (). This is comparison operator that compares the title
										*	of the passed in document, with the title set in this objects
										*	constructor.
										*
										*	@param		inDocument		The document to compare titles against.
										*
										*	@exception	none
										*	
										*	@return		bool			True if the titles match.
										*/
			bool						operator()( const CP_Document_Base * inDocument ) const
											{
												return( inDocument->GetFileName() == fDocumentTitle );
											}
			
private:
	CP_String		fDocumentTitle;
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Document_Base

