//=========================================================================
//
//	FILE:				CP_FileIterator.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class used to iterate and list files in a directory
//						in the underlying OSs file system. The user can supply
//						an optional CP_FileFilter object to control which files
//						are listed.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 6 $
//						Latest Checkout:  $Date: 2004-12-23 15:43:54 -0800 (Thu, 23 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_FileIterator
#define _H_CP_FileIterator
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileFilter.h"
#include "CP_FileSystemSpec.h"
#include "CP_Types.h"

	// Standard Library
#include <vector>
#include <iterator>

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_FileIterator
*	@brief		Class used to iterate and list files in a directory in the underlying OSs file system.
*
*				The user can supply an optional CP_FileFilter object to control which files are listed.
*/
#if TARGET_OS_LINUX
	class CP_EXPORT CP_FileIterator //: public std::iterator< std::forward_iterator_tag, CP_FileSystemSpec>
#else
class CP_EXPORT CP_FileIterator : public std::iterator< std::forward_iterator_tag, CP_FileSystemSpec>
#endif
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FileIterator();
										CP_FileIterator(
												const CP_FileSystemSpec& inParentDirectory,
												CP_FileFilter *inFilter = NULL );
										CP_FileIterator( const CP_FileIterator& inFilterIterator );
	//! Destructor.
	virtual								~CP_FileIterator() throw();
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Returns a CP_FileSystemSpec that represents the directory that this
										*	iterator is being used in currently.
										*	
										*	@exception	none
										*
										*	@return		CP_FileSystemSpec const		The directory being iterated over.
										*/
			CP_FileSystemSpec			GetDirectory() const								{ return( fParentDirectory ); }
	//@}

	//! @name File Specs List
	//@{
										/*!
										*	Returns a CP_FileSystemSpec_Array that contains a list of CP_FileSystemSpecs for
										*	each item in the directory that matches the filters criteria if one is installed,
										*	otherwise all file system objects are returned.
										*	
										*	@exception	none
										*
										*	@return		CP_FileSystemSpec_Array		The list of files matching the filters criteria.
										*/
			CP_FileSystemSpec_Array		GetFileSpecs()										{ return( fFileSpecs ); }
	//@}
			
	//! @name File Filter
	//@{
										/*!
										*	Returns the CP_FileFilter object used with this iterator. NULL is returned if
										*	this iterator is not using a file filter.
										*	
										*	@exception	none
										*
										*	@return		CP_FileFilter *				The file filter object used with this iterator.
										*/
			CP_FileFilter 				*GetFileFilter()									{ return( fFileFilter ); }

										/*!
										*	Sets the CP_FileFilter object to use with this iterator. When this object has a filter,
										*	the filter controls which files will be returned.
										*	
										*	@param		inFilter			The CP_FileFilter to use with this object. It is
										*									safe to pass NULL here, which indicates you want to
										*									not use any filter.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetFileFilter( CP_FileFilter *inFilter )			{ fFileFilter = inFilter; }
	//@}

protected:

	// class data
protected:
	CP_FileSystemSpec					fParentDirectory;		/*! The directory we are searching in. */
	CP_FileSystemSpec_Array				fFileSpecs;				/*! List of CP_FileSystemSpecs that match the filter, if any.
																	Otherwise, all files. */
	
	CP_FileFilter						*fFileFilter;			/*! CP_FileFilter object to control which files are listed. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
protected:
	//! @name Iteration
	//@{
	virtual	void						IterateDirectory( const CP_FileSystemSpec& inParentDirectory );
	//@}
#endif

#if TARGET_OS_WIN32
protected:
	//! @name Iteration
	//@{
	virtual	void						IterateDirectory( const CP_FileSystemSpec& inParentDirectory );
	//@}
#endif

#if TARGET_OS_LINUX
protected:
	//! @name Iteration
	//@{
	virtual	void						IterateDirectory( const CP_FileSystemSpec& inParentDirectory );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_FileIterator

