//=========================================================================
//
//	FILE:				CP_ScrollableView.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A CP_View class that knows how to handle scrolling.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 264 $
//						Latest Checkout:  $Date: 2005-08-28 19:28:53 -0700 (Sun, 28 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_ScrollableView
#define _H_CP_ScrollableView
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_DataValueHolder.h"
#include "CP_Rect.h"
#include "CP_Region.h"
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_ScrollableView
*	@brief		A CP_View class that knows how to handle scrolling
*
*	@ingroup	UI
*/
class CP_EXPORT CP_ScrollableView : public CP_View
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ScrollableView();
										CP_ScrollableView(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inCreateImp = true,
												UInt32 inFlags = CP_E_ViewFlags_None );
	//! Destructor.
	virtual								~CP_ScrollableView() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Image
	//@{
										/*!
										*	Returns the bounds of the image being displayed.
										*	
										*	@exception	none
										*	
										*	@return		CP_Rect const		Bounds of the image.
										*/
			CP_Rect						GetImageBounds() const										{ return( fImageBounds ); }
			void						SetImageBounds(
												const CP_Rect& inBounds,
												bool inBroadcastChange = true );

										/*!
										*	Returns the size of the image. This will be the width and height
										*	of the image bounds.
										*	
										*	@exception	none
										*	
										*	@return		CP_Size const		Bounds of the image.
										*/
			CP_Size						GetImageSize() const										{ return( CP_Size( fImageBounds.Width(), fImageBounds.Height() ) ); }

										/*!
										*	Returns the distance the view has been scrolled.
										*	
										*	@exception	none
										*	
										*	@return		CP_Point const		Distance the view has been scrolled.
										*/
			CP_Point					GetScrolledAmt() const										{ return( fScrolledAmt ); }

										/*!
										*	Sets the distance the view has been scrolled.
										*
										*	@param		inScrolledAmt		The distance the view has been scrolled.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetScrolledAmt( const CP_Point& inScrolledAmt )				{ fScrolledAmt = inScrolledAmt; }

										/*!
										*	Sets the amount the view is scrolled each time a line scroll message is
										*	received. This is caused by the user clicking in the arrows of a scroll bar.
										*
										*	@param		inScrollUnit		Distance to scroll when scrolling by lines.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetScrollUnit( const CP_Size& inScrollUnit );

										/*!
										*	Returns the distance to scroll when the user requests the view to be
										*	scrolled by a unit.
										*	
										*	@exception	none
										*	
										*	@return		CP_Size const		Distance to scroll.
										*/
			CP_Size						GetScrollUnit()	const										{ return( fScrollUnit ); }
	
			CP_Point					ImageToLocalPoint( const CP_Point& inImagePoint );
			CP_Point					LocalToImagePoint( const CP_Point& inLocalPoint );
	//@}

	//! @name Scrolling
	//@{
			void						ScrollBy(
												SInt32 inHorAmt,
												SInt32 inVerAmt );
			void						ScrollTo(
												SInt32 inHorLocation,
												SInt32 inVerLocation );
	//@}

	//! @name Signals
	//@{
			void						HandleHorScrollValue(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );
			void						HandleVerScrollValue(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );
	//@}

protected:
	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

	//! @name Scrolling
	//@{
	virtual	CP_Region					ScrollRect(
												const CP_Rect& inScrollArea,
												SInt32 inHorAmt,
												SInt32 inVerAmt );
	//@}

private:
	//! @name Scrolling
	//@{
			void						ScrollBits(
												SInt32 inHorAmt,
												SInt32 inVerAmt );

			void						LimitScrollAmts(
												SInt32& inHorAmt,
												SInt32& inVerAmt );
	//@}

			void						UpdatePortOrigin();

	// class data
protected:
	CP_Rect								fImageBounds;				/*! Bounnding rect of the image. */
	
	CP_Point							fScrolledAmt;				/*! Distance pane has been scrolled. */
	CP_Size								fScrollUnit;				/*! Distance to scroll each time. */

	CP_DataValueSignal					fImageBoundsChangedSignal;	/*! Signal sent when the image size changes. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_ScrollableView

